/*
 * ----------------------------------------------------------------------
 * WARNING: THIS IS A BETA RELEASE OF LAM/MPI.  USERS ARE ADVISED TO USE
 * IT AT THEIR OWN RISK.  SEE THE MAIN WEB SITE
 * (http://www.mpi.nd.edu/lam/) FOR STABLE VERSIONS OF LAM/MPI.
 * ----------------------------------------------------------------------
 * 
 * Copyright 1998-2000, University of Notre Dame.
 * Authors: Jeffrey M. Squyres with Kinis L. Meyer, M. D. McNally,
 *          and Andrew Lumsdaine
 * 
 * This file is part of the BETA RELEASE of the Notre Dame LAM
 * implementation of MPI.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame LAM implementation of MPI along with the software; see the file
 * LICENSE.  If not, contact Office of Research, University of Notre
 * Dame, Notre Dame, IN 46556.
 * 
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 * 
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.
 * 
 * Additional copyrights may follow.
 * 
 *
 *	$Id: allocmem.c,v 6.2 2000/08/10 17:36:12 jsquyres Exp $
 *
 *	Function:	- allocate memory for the RPI
 *	Accepts:	- size to allocate
 *			- info object
 *			- pointer to memory to allocate
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>

/*@

MPI_Alloc_mem - Allocate memory for use by MPI message passing.

Input Parameters:
. size - number of bytes to allocate
. info - 'MPI_Info' object

Output Parameter:
. baseptr - pointer to address of allocated memory

Notes:

'MPI_INFO_NULL' is always valid for this function.  For the TCP, SYSV,
and USYSV RPIs, the 'info' argument is ignored, and this function is
simply a wrapper around 'malloc'.

However, future RPIs (e.g., for Myrinet and VIA-based RPIs) will
provide "special" memory allocation routines that will allocate memory
that can be used directly for DMA.

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_ARG
.N MPI_ERR_NO_MEM

.N ACK
@*/
int MPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr)
{
  lam_initerr();
  lam_setfunc(BLKMPIALLOCMEM);
  
  if (size < 0) {
    return(lam_errfunc(MPI_COMM_WORLD,
		       BLKMPIALLOCMEM, lam_mkerr(MPI_ERR_ARG, 0)));
  }
  if (baseptr == 0) {
    return(lam_errfunc(MPI_COMM_WORLD,
		       BLKMPIALLOCMEM, lam_mkerr(MPI_ERR_ARG, 0)));
  }
  
  if (_rpi_c2c_alloc_mem(size, info, baseptr) != MPI_SUCCESS) {
    return(lam_errfunc(MPI_COMM_WORLD,
		       BLKMPIALLOCMEM, lam_mkerr(MPI_ERR_NO_MEM, 0)));
  }

  lam_resetfunc(BLKMPIALLOCMEM);
  return(MPI_SUCCESS);
}
