/*
 * ----------------------------------------------------------------------
 * WARNING: THIS IS A BETA RELEASE OF LAM/MPI.  USERS ARE ADVISED TO USE
 * IT AT THEIR OWN RISK.  SEE THE MAIN WEB SITE
 * (http://www.mpi.nd.edu/lam/) FOR STABLE VERSIONS OF LAM/MPI.
 * ----------------------------------------------------------------------
 * 
 * Copyright 1998-2000, University of Notre Dame.
 * Authors: Jeffrey M. Squyres with Kinis L. Meyer, M. D. McNally,
 *          and Andrew Lumsdaine
 * 
 * This file is part of the BETA RELEASE of the Notre Dame LAM
 * implementation of MPI.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame LAM implementation of MPI along with the software; see the file
 * LICENSE.  If not, contact Office of Research, University of Notre
 * Dame, Notre Dame, IN 46556.
 * 
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 * 
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.
 * 
 * Additional copyrights may follow.
 * 
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: infogetnth.c,v 6.4 1999/12/31 21:27:30 jsquyres Exp $
 *
 *	Function:	- get n'th key in info object
 *	Accepts:	- info
 *			- n
 *			- key (out)
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <string.h>

#include <all_list.h>
#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>


/*@

MPI_Info_get_nthkey - Get a key indexed by integer from an 'MPI_Info'
object

int MPI_Info_get_nthkey(MPI_Info info, int n, char *key)

Input Parameters:
+ info - info object (handle)
- n - index of key to retrieve (integer)

Output Parameter:
. key - character string of at least 'MPI_MAX_INFO_KEY' characters

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_ARG
@*/
int MPI_Info_get_nthkey(MPI_Info info, int n, char *key)
{
	struct _infoent	*p;			/* key/value pair */
	
	lam_initerr_m();
	lam_setfunc_m(BLKMPIINFOGETNTHKEY);

	if (info == 0 || n >= al_count(info->info_list) || key == 0) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIINFOGETNTHKEY,
					lam_mkerr(MPI_ERR_ARG, 0)));
	}

	for (p = al_top(info->info_list); n > 0; n--) {
		p = al_next(info->info_list, p);
	}

	strcpy(key, p->ie_key);

	lam_resetfunc_m(BLKMPIINFOGETNTHKEY);
	return(MPI_SUCCESS);
}
