/*
 * ----------------------------------------------------------------------
 * WARNING: THIS IS A BETA RELEASE OF LAM/MPI.  USERS ARE ADVISED TO USE
 * IT AT THEIR OWN RISK.  SEE THE MAIN WEB SITE
 * (http://www.mpi.nd.edu/lam/) FOR STABLE VERSIONS OF LAM/MPI.
 * ----------------------------------------------------------------------
 * 
 * Copyright 1998-2000, University of Notre Dame.
 * Authors: Jeffrey M. Squyres with Kinis L. Meyer, M. D. McNally,
 *          and Andrew Lumsdaine
 * 
 * This file is part of the BETA RELEASE of the Notre Dame LAM
 * implementation of MPI.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame LAM implementation of MPI along with the software; see the file
 * LICENSE.  If not, contact Office of Research, University of Notre
 * Dame, Notre Dame, IN 46556.
 * 
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 * 
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.
 * 
 * Additional copyrights may follow.
 * 
 *	Ohio Trollius
 *	Copyright 1996 The Ohio State University
 *	RBD
 *
 *	$Id: lamcid.c,v 6.6 2000/05/27 01:42:37 jsquyres Exp $
 *
 *	Function:	- manage context identifiers
 *			- trivial version
 */

#include <typical.h>
#include <mpi.h>
#include <mpisys.h>
#include <lam_config.h>

/*
 * static variables
 */
static int first = 1;
static unsigned char map[MPI_MAX_CID / (sizeof(char) * 8)];

/*
 *	lam_getcid
 *
 *	Function:	- get next available context ID
 *	Returns:	- context ID or LAMERROR
 */
int
lam_getcid()

{
  int byte, bit, cid;

  /* Ensure that the bitmap is initialized properly */

  if (first == 1) {
    lam_nukecids();
    first = 0;
  }

  /* See if everything is taken right up front */

  if (map[(MPI_MAX_CID / (sizeof(char) * 8)) - 1] >= (1 << 7))
    return LAMERROR;

  cid = 0;
  for (byte = (MPI_MAX_CID / (sizeof(char) * 8)) - 1; byte >= 0; byte--) {

    /* Catch the case where at least the MSB of the byte (i.e., the
       highest CID represented by this byte) is taken, so move on to
       the next byte (which must have alredy been zero, 'cause we
       passed it). */

    if (map[byte] >= (1 << 7)) {
      byte++;
      map[byte] = 1;
      cid = 8 * byte;
      break;
    } 

    /* Now the case where we must have some open bits on the MSB side
       of the byte; figure out what the largest one is */

    else if (map[byte] > 0) {
      for (bit = 7; bit >= 0; bit--)
	if ((map[byte] & (1 << bit)) != 0)
	  break;

      /* bit will always be >= 0 'cause of the outside if statement
	 (map[i] > 0), and will always be < 7 'cause of the first if
	 statement (>= (1 << 7) */
      
      bit++;
      map[byte] |= (1 << bit);
      cid = (8 * byte) + bit;
      break;
    }
  }

  /* Special case for MPI_COMM_WORLD */

  if (cid == 0)
    map[0] = 1;

  /* Return the calculated cidue */

  return cid;
}

/*
 *	lam_setcid
 *
 *	Function:	- set highest used context ID
 * Accepts: - context ID */
void
lam_setcid(cid)

int			cid;

{
  int byte, bit;
  
  byte = cid / 8;
  bit = (1 << (cid % 8));
  map[byte] |= bit;

#if LAM_WANT_IMPI
  cid++;
  byte = cid / 8;
  bit = (1 << (cid % 8));
  map[byte] |= bit;
#endif
}

/*
 *	lam_rmcid
 *
 *	Function:	- deallocate (remove) a context ID
 *	Accepts:	- context ID
 */
void
lam_rmcid(cid)

int			cid;

{
  int byte, bit;
  
  byte = cid / 8;
  bit = (1 << (cid % 8));

  map[byte] &= ~bit;
}

/*
 *	lam_nukecids
 *
 *	Function:	- deallocate all context IDs (cleanup)
 */
void
lam_nukecids()

{
  int i;
  for (i = 0; i < (MPI_MAX_CID / (sizeof(char) * 8)); i++)
    map[i] = 0;
}
