/*
 * ----------------------------------------------------------------------
 * WARNING: THIS IS A BETA RELEASE OF LAM/MPI.  USERS ARE ADVISED TO USE
 * IT AT THEIR OWN RISK.  SEE THE MAIN WEB SITE
 * (http://www.mpi.nd.edu/lam/) FOR STABLE VERSIONS OF LAM/MPI.
 * ----------------------------------------------------------------------
 * 
 * Copyright 1998-2000, University of Notre Dame.
 * Authors: Jeffrey M. Squyres with Kinis L. Meyer, M. D. McNally,
 *          and Andrew Lumsdaine
 * 
 * This file is part of the BETA RELEASE of the Notre Dame LAM
 * implementation of MPI.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame LAM implementation of MPI along with the software; see the file
 * LICENSE.  If not, contact Office of Research, University of Notre
 * Dame, Notre Dame, IN 46556.
 * 
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 * 
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.
 * 
 * Additional copyrights may follow.
 * 
 *	Ohio Trollius
 *	Copyright 1996 The Ohio State University
 *	RBD
 *
 *	$Id: topotest.c,v 6.3 1999/12/31 21:27:41 jsquyres Exp $
 *
 *	Function:	- get topology assigned to communicator
 *	Accepts:	- communicator
 *			- ptr topology type
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>

/*@

MPI_Topo_test - Determines the type of topology (if any) associated with a 
                communicator

Input Parameter:
. comm - communicator (handle) 

Output Parameter:
. ptopo - topology type of communicator 'comm' (choice).

Notes:

Upon successful completion, 'ptopo' will either be 'MPI_GRAPH',
'MPI_CART', or 'MPI_UNDEFINED'.

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_COMM
.N MPI_ERR_ARG

.seealso: MPI_Graph_create, MPI_Cart_create

.N ACK
@*/
int MPI_Topo_test(MPI_Comm comm, int *ptopo)
{
	lam_initerr();
	lam_setfunc(BLKMPITOPOTEST);
/*
 * Check the arguments.
 */
	if (comm == MPI_COMM_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITOPOTEST, lam_mkerr(MPI_ERR_COMM, 0)));
	}

	if (ptopo == 0) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITOPOTEST, lam_mkerr(MPI_ERR_ARG, 0)));
	}

	*ptopo = comm->c_topo_type;

	lam_resetfunc(BLKMPITOPOTEST);
	return(MPI_SUCCESS);
}
