/*
 * ----------------------------------------------------------------------
 * WARNING: THIS IS A BETA RELEASE OF LAM/MPI.  USERS ARE ADVISED TO USE
 * IT AT THEIR OWN RISK.  SEE THE MAIN WEB SITE
 * (http://www.mpi.nd.edu/lam/) FOR STABLE VERSIONS OF LAM/MPI.
 * ----------------------------------------------------------------------
 * 
 * Copyright 1998-2000, University of Notre Dame.
 * Authors: Jeffrey M. Squyres with Kinis L. Meyer, M. D. McNally,
 *          and Andrew Lumsdaine
 * 
 * This file is part of the BETA RELEASE of the Notre Dame LAM
 * implementation of MPI.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame LAM implementation of MPI along with the software; see the file
 * LICENSE.  If not, contact Office of Research, University of Notre
 * Dame, Notre Dame, IN 46556.
 * 
 * Permission to modify the code and to distribute modified code is
 * granted, provided the text of this NOTICE is retained, a notice that
 * the code was modified is included with the above COPYRIGHT NOTICE and
 * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
 * file is distributed with the modified code.
 * 
 * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
 * By way of example, but not limitation, Licensor MAKES NO
 * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
 * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
 * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
 * OR OTHER RIGHTS.
 * 
 * Additional copyrights may follow.
 * 
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	GDB/RBD/NJN
 *
 *	$Id: hf77.c,v 6.29 2000/07/05 18:03:59 jsquyres Exp $
 *
 *	Function:	- wrapper for FORTRAN program compilation
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <lam_config.h>
#include <args.h>
#include <boot.h>
#include <portable.h>

#ifndef DEFTF77
#define DEFTF77 LAM_F77
#endif

/*
 * external functions
 */
extern int		cnfexec();

/*
 * local variables
 */
static char		ph[4][LAM_PATH_MAX];
static char		plib[3][LAM_PATH_MAX];
static char		extra[LAM_PATH_MAX];

int
main(argc, argv)

int			argc;
char			*argv[];

{
	int		ac;		/* f77 # args */
	int		status;		/* f77 return code */
	int		fl_libs;	/* add libs to command line */
	int		fl_show;	/* show what would be done */
	int             fl_building;    /* are we building LAM right now? */
	int             fl_profile;     /* want profile lib? */
	int		i;		/* favourite index */
	int		num_incs;	/* number of -I's */
	int		num_libs;	/* number of -L's */
	char		**av;		/* f77 arg vector */
	char		**sys;		/* system libraries arg vector */
	char		*inchome;	/* include base dir */
	char		*libhome;	/* library base dir */
	char		*tf77;		/* f77 tool */
#if LAM_WANT_ROMIO || LAM_WANT_MPI2CPP
	int             fd;
#endif
#if LAM_WANT_PROFILE
	int             added_lpmpi = 0;
#endif

	if (strcmp(DEFTF77, "/bin/false") == 0) {
	  fprintf(stderr, "Warning: it appears that LAM was configured with the --without-fc option.\n");
	  fprintf(stderr, "Warning: LAM/MPI compilation of Fortran programs may fail!\n");
	}

/*
 * LAM_TOP_BUILDDIR and LAM_TOP_SRCDIR should only be used internally
 * -- it is *not* a user-level flag (and as such, is not documented
 * :-)
 */
	libhome = getenv("LAM_TOP_BUILDDIR");
	if (libhome != 0) {
	  inchome = getenv("LAM_TOP_SRCDIR");
	  if (inchome == 0) {
	    show_help("compile", "internal_build", NULL);
	    exit(1);
	  }
	} else {
	  libhome = getenv("LAMHOME");
	  if (libhome == 0) libhome = getenv("TROLLIUSHOME");
	  if (libhome == 0) libhome = LAM_PREFIX;

	  inchome = libhome;
	}
/*
 * Command line options
 */
	fl_show = 0;
	fl_libs = 1;
	fl_building = 0;
	fl_profile = 0;

	for (i = 1; i < argc; ++i) {
		if (strcmp(argv[i], "-showme") == 0) {
			fl_show = 1;
		} else if (strcmp(argv[i], "-c") == 0) {
			fl_libs = 0;
		} else if (strcmp(argv[i], "-lam-building") == 0) {
			fl_building = 1;
		} else if (strcmp(argv[i], "-lpmpi") == 0) {
			fl_profile = 1;
		}
	}
#if !LAM_WANT_PROFILE
/*
 * Sanity check
 */
	if (fl_profile == 1) {
	  show_help("compile", "no-profiling-support", argv[0], NULL);
	  fl_profile = 0;
	}
#endif

	if ((strlen(inchome) + 32) >= LAM_PATH_MAX ||
	    (strlen(libhome) + 32) >= LAM_PATH_MAX) {
	  char buffer[32];

	  snprintf(buffer, 32, "%d", LAM_PATH_MAX - 32);
	  if (strlen(inchome) + 32 >= LAM_PATH_MAX)
	    show_help("compile", "phome-too-long", inchome, buffer, "hcc", 
		      NULL);
	  else
	    show_help("compile", "phome-too-long", libhome, buffer, "hcc", 
		      NULL);
	  exit(1);
	}
/*
 * Form include directory pathname.
 * If we're building LAM:
 *
 * - if it's a VPATH build, lam_config.h will be in
 * LAM_TOP_BUILDDIR/share/include, and the rest will be in
 * LAM_TOP_SRCDIR/share/include.  Unfortunately, we have to worry
 * about the mpi2c++ header files (since <mpi.h> may include it), so
 * we have ot put in LAM_TOP_SRCDIR/mpi2c++/src/mpi2c++, and are
 * referenced through LAM_TOPSRCDIR/mpi2c++/src.
 *
 * - if it's not a VPATH build, everything will be in
 * LAM_TOP_SRCDIR/share/include.  So it doesn't hurt to list both
 * (from above); it simplifies the logic.
 *
 * If we're not building LAM, everything is in $prefix/include 
 */
	num_incs = 1;
	strcpy(ph[0], "-I");
	strcat(ph[0], inchome);
	strcat(ph[0], STRSDIR);
	if (fl_building) {
	  strcat(ph[0], "share");
	  strcat(ph[0], STRSDIR);
	  strcat(ph[0], "include");

	  strcpy(ph[1], "-I");
	  strcat(ph[1], inchome);
	  strcat(ph[1], STRSDIR);
	  strcat(ph[1], "mpi2c++");
	  strcat(ph[1], STRSDIR);
	  strcat(ph[1], "src");

	  strcpy(ph[2], "-I");
	  strcat(ph[2], inchome);
	  strcat(ph[2], STRSDIR);
	  strcat(ph[2], "mpi2c++");
	  strcat(ph[2], STRSDIR);
	  strcat(ph[2], "src");
	  strcat(ph[2], STRSDIR);
	  strcat(ph[2], "mpi2c++");

	  strcpy(ph[3], "-I");
	  strcat(ph[3], libhome);
	  strcat(ph[3], STRSDIR);
	  strcat(ph[3], "share");
	  strcat(ph[3], STRSDIR);
	  strcat(ph[3], "include");
	  num_incs = 4;
	} else
	  strcat(ph[0], "include");
/*
 * Form library directory pathname.
 *
 * If we're building LAM, use "-LLAM_TOP_BUILDDIR/share
 * -LLAM_TOP_BUILDDIR/share/pmpi".
 *
 * Otherwise, use "-L$prefix/lib".
 */
	num_libs = 1;
	strcpy(plib[0], LAM_FLIBPATH);
	strcat(plib[0], libhome);
	strcat(plib[0], STRSDIR);
	if (fl_building) {
#if LAM_WANT_PROFILE
	  strcpy(plib[1], LAM_FLIBPATH);
	  strcat(plib[1], libhome);
	  strcat(plib[1], STRSDIR);
	  strcat(plib[1], "share");
	  strcat(plib[1], STRSDIR);
	  strcat(plib[1], "pmpi");

	  strcpy(plib[2], LAM_FLIBPATH);
	  strcat(plib[2], libhome);
	  strcat(plib[2], STRSDIR);
#endif
	  strcat(plib[2], "share");
	  num_libs = 3;
	} else
	  strcat(plib[0], "lib");

	tf77 = getenv("LAMHF77");
	if (tf77 == 0) tf77 = getenv("TROLLIUSHF77");
	if (tf77 == 0) tf77 = DEFTF77;

	if (strcmp(tf77, "false") == 0 || strlen(tf77) == 0) {
	  show_help("hf77", "no-fortran-support", NULL);
	  exit(1);
	}

	if ((av = sfh_argv_break(tf77, ' ')) == 0) {
	  show_help(NULL, "lib-call-fail", "sfh_argv_break", NULL);
	  exit(errno);
	}

	ac = sfh_argv_count(av);
/*
 * If we're building LAM right now, #define LAM_BUILDING to be 1 so
 * that we don't try to include mpi++.h and mpio.h in mpi.h
 */
	if (fl_building) {
	  strcpy(extra, "-DLAM_BUILDING=1");
	  sfh_argv_add(&ac, &av, extra);
	}

	for (i = 0; i < num_incs; i++)
	  sfh_argv_add(&ac, &av, ph[i]);
	if (fl_libs) {
	  for (i = 0; i < num_libs; i++)
	    sfh_argv_add(&ac, &av, plib[i]);
	}

/*
 * Don't copy -showme or -lam-building, and -lpmpi was already insertted
 */
	for (i = 1; i < argc; ++i) {
		if (strcmp(argv[i], "-showme") &&
		    strcmp(argv[i], "-lam-building") &&
		    strcmp(argv[i], "-lpmpi")) {
			sfh_argv_add(&ac, &av, argv[i]);
		}
	}

	if (fl_libs) {
/*
 * Add system dependent link arguments.
 */
		if (strlen(LAM_HBIND) > 0) {
			if ((sys = sfh_argv_break(LAM_HBIND, ' ')) == 0) {
			  show_help(NULL, "lib-call-fail", "sfh_argv_break", 
				    NULL);
			  exit(errno);
			}

			for ( ; *sys; sys++) {
				sfh_argv_add(&ac, &av, *sys);
			}
		}
		if (fl_building == 0) {
/*
 * Do we want MPI 2 C++ bindings support?
 *
 * Tricky -- we have to test for presence of libmpi++.a first, "just
 * in case".  This is really only here for the obscure case where a
 * fortran module calls some other module that calls something in the
 * C++ bindings (because I don't know of any way to call C++ directly
 * from Fortran!).  
 */
#if LAM_WANT_MPI2CPP
		  strcpy(extra, plib[0] + 2);
		  strcat(extra, "/liblammpi++.a");
		  fd = open(extra, O_RDONLY);
		  if (fd >= 0) {
		    sfh_argv_add(&ac, &av, "-llammpi++");
		    close(fd);
		  }
#endif
/*
 * Do we want ROMIO support?
 * Tricky -- we have to test for presence of libmpio.a first, "just in 
 * case".
 */
#if LAM_WANT_ROMIO
		  strcpy(extra, plib[0] + 2);
		  strcat(extra, "/liblammpio.a");
		  fd = open(extra, O_RDONLY);
		  if (fd >= 0) {
		    sfh_argv_add(&ac, &av, "-llammpio");
		    /* Don't have to check for LAM_WANT_PROFILE here; we
		       can't have LAM_WANT_ROMIO without it */
		    sfh_argv_add(&ac, &av, "-lpmpi");
		    added_lpmpi = 1;
#if HAVE_LIBAIO
		    sfh_argv_add(&ac, &av, "-laio");
#endif
		    close(fd);
		  }
#endif
		}
/*
 * Add standard LAM libraries.
 */
#if LAM_WANT_PROFILE
		if (fl_profile && !added_lpmpi)
		  sfh_argv_add(&ac, &av, "-lpmpi");
#endif
#if LAM_WANT_FORTRAN
		  sfh_argv_add(&ac, &av, "-llamf77mpi");
#endif
		sfh_argv_add(&ac, &av, "-lmpi");
		sfh_argv_add(&ac, &av, "-llam");
/*
 * Add system dependent libraries.
 */
		if (strlen(LAM_SYSLIBS) > 0) {
			if ((sys = sfh_argv_break(LAM_SYSLIBS, ' ')) == 0) {
			  show_help(NULL, "lib-call-fail", "sfh_argv_break", 
				    NULL);
			  exit(errno);
			}

			for ( ; *sys; sys++) {
				sfh_argv_add(&ac, &av, *sys);
			}
		}
	}

	if (fl_show) {
		for (i = 0; i < ac; i++) {
			printf("%s ", av[i]);
		}
		printf("\n");
	} else {
		status = cnfexec(av);

		if (status) {
		  if (errno != 0)
		    perror("hf77");
		  exit(status);
		}
	}

	return(0);
}
